@@ -17,6 +17,8 @@
  */
 
 #include "gtd-manager.h"
+#include "gtd-storage-goa.h"
+#include "gtd-storage-local.h"
 #include "gtd-storage.h"
 #include "gtd-task.h"
 #include "gtd-task-list.h"
@@ -83,6 +85,7 @@ enum
   LIST_ADDED,
   LIST_CHANGED,
   LIST_REMOVED,
+  SHOW_ERROR_MESSAGE,
   STORAGE_ADDED,
   STORAGE_CHANGED,
   STORAGE_REMOVED,
@@ -100,7 +103,7 @@ enum
 
 static guint signals[NUM_SIGNALS] = { 0, };
 
-TaskData*
+static TaskData*
 task_data_new (GtdManager *manager,
                gpointer   *data)
 {
@@ -113,7 +116,7 @@ task_data_new (GtdManager *manager,
   return tdata;
 }
 
-gboolean
+static gboolean
 is_today (GDateTime *dt)
 {
   GDateTime *today;
@@ -132,9 +135,21 @@ is_today (GDateTime *dt)
   return FALSE;
 }
 
+static void
+emit_show_error_message (GtdManager  *manager,
+                         const gchar *primary_text,
+                         const gchar *secondary_text)
+{
+  g_signal_emit (manager,
+                 signals[SHOW_ERROR_MESSAGE],
+                 0,
+                 primary_text,
+                 secondary_text);
+}
+
 static void
 gtd_manager__setup_url (GtdManager *manager,
-                        GtdStorage *storage)
+                        GtdStorageGoa *storage)
 {
   GtdManagerPrivate *priv;
   GList *sources;
@@ -156,7 +171,6 @@ gtd_manager__setup_url (GtdManager *manager,
 
   for (l = sources; l != NULL; l = l->next)
     {
-      const gchar *parent_uid;
       ESource *source;
       ESource *goa_source;
 
@@ -172,9 +186,9 @@ gtd_manager__setup_url (GtdManager *manager,
           goa_ext = e_source_get_extension (goa_source, E_SOURCE_EXTENSION_GOA);
 
           /* Found an ESourceGoa with the same uid of storage */
-          if (g_strcmp0 (e_source_goa_get_account_id (goa_ext), gtd_storage_get_id (storage)) == 0)
+          if (g_strcmp0 (e_source_goa_get_account_id (goa_ext), gtd_storage_get_id (GTD_STORAGE (storage))) == 0)
             {
-              gtd_storage_set_parent (storage, e_source_get_uid (source));
+              gtd_storage_goa_set_parent (storage, e_source_get_uid (goa_source));
               break;
             }
         }
@@ -274,15 +288,12 @@ gtd_manager__goa_account_added_cb (GoaClient  *client,
 
       default_location = g_settings_get_string (priv->settings, "storage-location");
 
-      storage = gtd_storage_new (goa_account_get_id (account),
-                                 goa_account_get_provider_type (account),
-                                 goa_account_get_provider_name (account),
-                                 goa_account_get_identity (account));
+      storage = gtd_storage_goa_new (object);
 
       gtd_storage_set_enabled (storage, !goa_account_get_calendar_disabled (account));
       gtd_storage_set_is_default (storage, g_strcmp0 (gtd_storage_get_id (storage), default_location) == 0);
 
-      gtd_manager__setup_url (manager, storage);
+      gtd_manager__setup_url (manager, GTD_STORAGE_GOA (storage));
 
       priv->storage_locations = g_list_insert_sorted (priv->storage_locations,
                                                       storage,
@@ -326,21 +337,20 @@ gtd_manager__goa_client_finish_cb (GObject      *client,
           GoaAccount *account;
 
           object = l->data;
-          account = goa_object_get_account (object);
+          account = goa_object_peek_account (object);
 
           if (g_strv_contains (supported_providers, goa_account_get_provider_type (account)))
             {
               GtdStorage *storage;
 
-              storage = gtd_storage_new (goa_account_get_id (account),
-                                         goa_account_get_provider_type (account),
-                                         goa_account_get_provider_name (account),
-                                         goa_account_get_identity (account));
-
+              /* Create the new GOA storage */
+              storage = gtd_storage_goa_new (object);
               gtd_storage_set_enabled (storage, !goa_account_get_calendar_disabled (account));
               gtd_storage_set_is_default (storage, g_strcmp0 (gtd_storage_get_id (storage), default_location) == 0);
 
-              gtd_manager__setup_url (GTD_MANAGER (user_data), storage);
+              g_assert (gtd_storage_goa_get_account (GTD_STORAGE_GOA (storage)) == account);
+
+              gtd_manager__setup_url (GTD_MANAGER (user_data), GTD_STORAGE_GOA (storage));
 
               priv->storage_locations = g_list_insert_sorted (priv->storage_locations,
                                                               storage,
@@ -376,6 +386,10 @@ gtd_manager__goa_client_finish_cb (GObject      *client,
                  _("Error loading GNOME Online Accounts"),
                  error->message);
 
+      emit_show_error_message (GTD_MANAGER (user_data),
+                               _("Error loading GNOME Online Accounts"),
+                               error->message);
+
       g_clear_error (&error);
     }
 
@@ -388,7 +402,6 @@ gtd_manager__commit_source_finished (GObject      *registry,
                                      GAsyncResult *result,
                                      gpointer      user_data)
 {
-  GtdManagerPrivate *priv = GTD_MANAGER (user_data)->priv;
   GError *error = NULL;
 
   g_return_if_fail (GTD_IS_MANAGER (user_data));
@@ -405,40 +418,95 @@ gtd_manager__commit_source_finished (GObject      *registry,
                  _("Error saving task list"),
                  error->message);
 
+      emit_show_error_message (GTD_MANAGER (user_data),
+                               _("Error saving task list"),
+                               error->message);
+
       g_error_free (error);
       return;
     }
 }
 
+static void
+task_list_removal_finished (GtdManager  *manager,
+                            ESource     *source,
+                            GError     **error)
+{
+  gtd_object_set_ready (GTD_OBJECT (manager), TRUE);
+
+  if (*error)
+    {
+      g_warning ("%s: %s: %s",
+                 G_STRFUNC,
+                 _("Error removing task list"),
+                 (*error)->message);
+
+      emit_show_error_message (manager,
+                               _("Error removing task list"),
+                               (*error)->message);
+
+      g_clear_error (error);
+    }
+}
+
+static void
+gtd_manager__remote_create_source_finished (GObject      *source,
+                                            GAsyncResult *result,
+                                            gpointer      user_data)
+{
+  GError *error;
+
+  error = NULL;
+
+  e_source_remote_create_finish (E_SOURCE (source),
+                                 result,
+                                 &error);
+
+  if (error)
+    {
+      g_warning ("%s: %s: %s",
+                 G_STRFUNC,
+                 _("Error creating task list"),
+                 error->message);
+
+      emit_show_error_message (GTD_MANAGER (user_data),
+                               _("Error creating task list"),
+                               error->message);
+
+      g_clear_error (&error);
+    }
+}
+
+static void
+gtd_manager__remote_delete_finished (GObject      *source,
+                                     GAsyncResult *result,
+                                     gpointer      user_data)
+{
+  GError *error = NULL;
+
+  e_source_remote_delete_finish (E_SOURCE (source),
+                                 result,
+                                 &error);
+
+  task_list_removal_finished (GTD_MANAGER (user_data),
+                              E_SOURCE (source),
+                              &error);
+}
+
 static void
 gtd_manager__remove_source_finished (GObject      *source,
                                      GAsyncResult *result,
                                      gpointer      user_data)
 {
-  GtdManagerPrivate *priv = GTD_MANAGER (user_data)->priv;
   GError *error = NULL;
 
-  g_return_if_fail (GTD_IS_MANAGER (user_data));
-
-  gtd_object_set_ready (GTD_OBJECT (user_data), TRUE);
   e_source_remove_finish (E_SOURCE (source),
                           result,
                           &error);
 
-  if (error)
-    {
-      g_warning ("%s: %s: %s",
-                 G_STRFUNC,
-                 _("Error removing task list"),
-                 error->message);
-
-      g_error_free (error);
-      return;
-    }
-  else
-    {
-      priv->task_lists = g_list_remove (priv->task_lists, source);
-    }
+  task_list_removal_finished (GTD_MANAGER (user_data),
+                              E_SOURCE (source),
+                              &error);
 }
 
 static void
@@ -448,15 +516,14 @@ gtd_manager__create_task_finished (GObject      *client,
 {
   GtdManagerPrivate *priv;
   TaskData *data = user_data;
-  gboolean success;
   gchar *new_uid = NULL;
   GError *error = NULL;
 
   priv = data->manager->priv;
-  success = e_cal_client_create_object_finish (E_CAL_CLIENT (client),
-                                               result,
-                                               &new_uid,
-                                               &error);
+  e_cal_client_create_object_finish (E_CAL_CLIENT (client),
+                                     result,
+                                     &new_uid,
+                                     &error);
 
   gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
 
@@ -467,6 +534,10 @@ gtd_manager__create_task_finished (GObject      *client,
                  _("Error creating task"),
                  error->message);
 
+      emit_show_error_message (GTD_MANAGER (user_data),
+                               _("Error creating task"),
+                               error->message);
+
       g_error_free (error);
       g_free (data);
       return;
@@ -507,13 +578,12 @@ gtd_manager__remove_task_finished (GObject      *client,
 {
   GtdManagerPrivate *priv;
   TaskData *data = user_data;
-  gboolean success;
   GError *error = NULL;
 
   priv = data->manager->priv;
-  success = e_cal_client_remove_object_finish (E_CAL_CLIENT (client),
-                                               result,
-                                               &error);
+  e_cal_client_remove_object_finish (E_CAL_CLIENT (client),
+                                     result,
+                                     &error);
 
   gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
 
@@ -531,6 +601,10 @@ gtd_manager__remove_task_finished (GObject      *client,
                  _("Error removing task"),
                  error->message);
 
+      emit_show_error_message (GTD_MANAGER (user_data),
+                               _("Error removing task"),
+                               error->message);
+
       g_error_free (error);
       return;
     }
@@ -545,14 +619,13 @@ gtd_manager__update_task_finished (GObject      *client,
   GDateTime *dt;
   TaskData *data = user_data;
   GtdTask *task;
-  gboolean success;
   GError *error = NULL;
 
   priv = data->manager->priv;
   task = GTD_TASK (data->data);
-  success = e_cal_client_modify_object_finish (E_CAL_CLIENT (client),
-                                               result,
-                                               &error);
+  e_cal_client_modify_object_finish (E_CAL_CLIENT (client),
+                                     result,
+                                     &error);
 
   /* Check if the task still fits internal lists */
   dt = gtd_task_get_due_date (task);
@@ -572,11 +645,6 @@ gtd_manager__update_task_finished (GObject      *client,
       gtd_task_list_remove_task (priv->today_tasks_list, task);
     }
 
-  g_clear_pointer (&dt, g_date_time_unref);
-  g_free (data);
-
-  gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
-
   if (error)
     {
       g_warning ("%s: %s: %s",
@@ -584,9 +652,17 @@ gtd_manager__update_task_finished (GObject      *client,
                  _("Error updating task"),
                  error->message);
 
+      emit_show_error_message (data->manager,
+                               _("Error updating task"),
+                               error->message);
+
       g_error_free (error);
-      return;
     }
+
+  g_clear_pointer (&dt, g_date_time_unref);
+  g_free (data);
+
+  gtd_object_set_ready (GTD_OBJECT (data->data), TRUE);
 }
 
 static void
@@ -717,11 +793,8 @@ gtd_manager__fill_task_list (GObject      *client,
 
   if (!error)
     {
-      GDateTime *today;
       GSList *l;
 
-      today = g_date_time_new_now_local ();
-
       for (l = component_list; l != NULL; l = l->next)
         {
           GDateTime *dt;
@@ -755,6 +828,10 @@ gtd_manager__fill_task_list (GObject      *client,
                  _("Error fetching tasks from list"),
                  error->message);
 
+      emit_show_error_message (GTD_MANAGER (user_data),
+                               _("Error fetching tasks from list"),
+                               error->message);
+
       g_error_free (error);
       return;
     }
@@ -829,6 +906,10 @@ gtd_manager__on_client_connected (GObject      *source_object,
                e_source_get_uid (source),
                error->message);
 
+      emit_show_error_message (GTD_MANAGER (user_data),
+                               _("Failed to connect to task list source"),
+                               error->message);
+
       g_error_free (error);
       return;
     }
@@ -978,8 +1059,6 @@ gtd_manager_get_property (GObject    *object,
                           GValue     *value,
                           GParamSpec *pspec)
 {
-  GtdManager *self = GTD_MANAGER (object);
-
   switch (prop_id)
     {
     default:
@@ -993,8 +1072,6 @@ gtd_manager_set_property (GObject      *object,
                           const GValue *value,
                           GParamSpec   *pspec)
 {
-  GtdManager *self = GTD_MANAGER (object);
-
   switch (prop_id)
     {
     default:
@@ -1025,10 +1102,7 @@ gtd_manager_constructed (GObject *object)
                          object);
 
   /* local storage location */
-  local_storage = gtd_storage_new ("local",
-                                   "local",
-                                   _("Local"),
-                                   _("On This Computer"));
+  local_storage = gtd_storage_local_new ();
   gtd_storage_set_enabled (local_storage, TRUE);
   gtd_storage_set_is_default (local_storage, g_strcmp0 (default_location, "local") == 0);
 
@@ -1061,8 +1135,8 @@ gtd_manager_class_init (GtdManagerClass *klass)
         object_class,
         PROP_GOA_CLIENT,
         g_param_spec_object ("goa-client",
-                            _("The online accounts client of the manager"),
-                            _("The read-only GNOME online accounts client loaded and owned by the manager"),
+                            "The online accounts client of the manager",
+                            "The read-only GNOME online accounts client loaded and owned by the manager",
                             GOA_TYPE_CLIENT,
                             G_PARAM_READABLE));
 
@@ -1075,8 +1149,8 @@ gtd_manager_class_init (GtdManagerClass *klass)
         object_class,
         PROP_GOA_CLIENT_READY,
         g_param_spec_boolean ("goa-client-ready",
-                              _("Whether GNOME Online Accounts client is ready"),
-                              _("Whether the read-only GNOME online accounts client is loaded"),
+                              "Whether GNOME Online Accounts client is ready",
+                              "Whether the read-only GNOME online accounts client is loaded",
                               FALSE,
                               G_PARAM_READABLE));
 
@@ -1089,8 +1163,8 @@ gtd_manager_class_init (GtdManagerClass *klass)
         object_class,
         PROP_SOURCE_REGISTRY,
         g_param_spec_object ("source-registry",
-                            _("The source registry of the manager"),
-                            _("The read-only source registry loaded and owned by the manager"),
+                            "The source registry of the manager",
+                            "The read-only source registry loaded and owned by the manager",
                             E_TYPE_SOURCE_REGISTRY,
                             G_PARAM_READABLE));
 
@@ -1164,6 +1238,25 @@ gtd_manager_class_init (GtdManagerClass *klass)
                                         1,
                                         GTD_TYPE_TASK_LIST);
 
+  /**
+   * GtdManager::show-error-message:
+   *
+   * Notifies about errors, and sends the error message for widgets
+   * to display.
+   *
+   */
+  signals[SHOW_ERROR_MESSAGE] = g_signal_new ("show-error-message",
+                                              GTD_TYPE_MANAGER,
+                                              G_SIGNAL_RUN_LAST,
+                                              0,
+                                              NULL,
+                                              NULL,
+                                              NULL,
+                                              G_TYPE_NONE,
+                                              2,
+                                              G_TYPE_STRING,
+                                              G_TYPE_STRING);
+
   /**
    * GtdManager::storage-added:
    *
@@ -1357,6 +1450,39 @@ gtd_manager_update_task (GtdManager *manager,
                               data);
 }
 
+/**
+ * gtd_manager_create_task_list:
+ *
+ *
+ * Creates a new task list at the given source.
+ *
+ * Returns:
+ */
+void
+gtd_manager_create_task_list (GtdManager  *manager,
+                              GtdTaskList *list)
+{
+  GtdManagerPrivate *priv;
+  ESource *source;
+  ESource *parent;
+
+  g_return_if_fail (GTD_IS_MANAGER (manager));
+  g_return_if_fail (GTD_IS_TASK_LIST (list));
+  g_return_if_fail (gtd_task_list_get_source (list));
+
+  priv = manager->priv;
+  source = gtd_task_list_get_source (list);
+  parent = e_source_registry_ref_source (priv->source_registry, e_source_get_parent (source));
+
+  e_source_remote_create (parent,
+                          source,
+                          NULL,
+                          (GAsyncReadyCallback) gtd_manager__remote_create_source_finished,
+                          manager);
+
+  g_object_unref (parent);
+}
+
 /**
  * gtd_manager_remove_task_list:
  * @manager: a #GtdManager
@@ -1379,10 +1505,26 @@ gtd_manager_remove_task_list (GtdManager  *manager,
   source = gtd_task_list_get_source (list);
 
   gtd_object_set_ready (GTD_OBJECT (manager), FALSE);
-  e_source_remove (source,
-                   NULL,
-                   (GAsyncReadyCallback) gtd_manager__remove_source_finished,
-                   manager);
+
+  if (e_source_get_remote_deletable (source))
+    {
+      e_source_remote_delete (source,
+                              NULL,
+                              (GAsyncReadyCallback) gtd_manager__remote_delete_finished,
+                              manager);
+    }
+  else
+    {
+      e_source_remove (source,
+                       NULL,
+                       (GAsyncReadyCallback) gtd_manager__remove_source_finished,
+                       manager);
+    }
+
+  g_signal_emit (manager,
+                 signals[LIST_REMOVED],
+                 0,
+                 list);
 }
 
 /**
@@ -1495,7 +1637,6 @@ gtd_manager_get_default_storage (GtdManager *manager)
 {
   GtdManagerPrivate *priv;
   GtdStorage *storage;
-  GList *l;
   gchar *storage_id;
 
   g_return_val_if_fail (GTD_IS_MANAGER (manager), NULL);
@@ -1549,6 +1690,22 @@ gtd_manager_set_default_storage (GtdManager *manager,
     }
 }
 
+/**
+ * gtd_manager_get_settings:
+ * @manager: a #GtdManager
+ *
+ * Retrieves the internal #GSettings from @manager.
+ *
+ * Returns: (transfer none): the internal #GSettings of @manager
+ */
+GSettings*
+gtd_manager_get_settings (GtdManager *manager)
+{
+  g_return_val_if_fail (GTD_IS_MANAGER (manager), NULL);
+
+  return manager->priv->settings;
+}
+
 /**
  * gtd_manager_get_is_first_run:
  * @manager: a #GtdManager
